from datetime import datetime
from client_ip import ClientIp


def _send_log_details(app_id, status_code, request, start_time, request_body):
    client_ip = ClientIp()
    request_method = request.method
    dct = {}
    dct["status_code"] = status_code
    dct["method"] = request_method
    dct["size"] = 0  # len(response.data)
    dct["path"] = request.path
    dct["args"] = request.args.to_dict() if request.args else {}
    dct["duration"] = datetime.utcnow().timestamp() - start_time
    dct["product_id"] = "tier"
    dct["app_id"] = app_id
    dct["timestamp"] = start_time
    dct["ip"] = client_ip.get_client_address(request.environ)
    dct["body"] = request_body
    dct["user_id"] = request.args.get("apikey") if "apikey" not in request.args else ""
    dct["headers"] = (
        {k: v for k, v in request.headers.items()} if request.headers else {}
    )
    # _send_details(dct)



from __future__ import absolute_import, division, print_function

import os

# Archetype Python
# API docs at http://docs.archetype.dev
# Authors:
# Behailu Tekletsadik : behailu@archetype.dev

# Configuration variables

secret_key = os.environ.get("ARCHETYPE_SECRET_KEY")
app_id = os.environ.get("ARCHETYPE_APP_ID")
prod_api_base = os.environ.get("ARCHETYPE_API_PROD_URL", "https://api.archetype.dev")
upload_api_base = "https://files.archetype.dev"
auth_version = 4
record_auth_requests = True
api_version = None
verify_ssl_certs = True
proxy = None
default_http_client = None
app_info = None
enable_telemetry = True
max_network_retries = 0

# Set to either 'debug' or 'info', controls console logging
log = None

from archetypesdk.api_resources import *


def set_app_info(name, partner_id=None, url=None, version=None):
    global app_info
    app_info = {
        "name": name,
        "partner_id": partner_id,
        "url": url,
        "version": version,
    }



from __future__ import absolute_import, division, print_function
from typing import List

from archetypesdk.api_requestor import APIRequestor
from archetypesdk.enums import Method

api_requestor = APIRequestor()
class CreatableAPIResource:
    @classmethod
    def Create(cls, version: int = 1, **params):
        object_name = cls.OBJECT_NAME
        path = f"/api/v{version}/create-{object_name}"
        api_resource = api_requestor.create_request(
            request_method=Method.POST,
            path=path,
            object=object_name,
            data=params,
            intent=f"Create {object_name}"
        )

        return api_resource


class RetrievableAPIResource:
    @classmethod
    def Retrieve(
        cls,
        id: str,
        version: int = 1,
    ):
        object_name = cls.OBJECT_NAME
        path = f"/api/v{version}/{object_name}/{id}"
        api_resource = api_requestor.create_request(
            request_method=Method.GET,
            path=path,
            object=object_name,
            intent=f"Retrieve {object_name}: {id}",
        )
        return api_resource


class ListableAPIResource:
    @classmethod
    def All(
        cls,
        version: int = 1,
    ):
        path = f"/api/v{version}/{cls.OBJECT_NAME}s"
        api_requestor = APIRequestor()
        billable_metrics = api_requestor.create_request(
            request_method=Method.GET,
            path=path,
            object=cls.OBJECT_NAME,
            intent=f"Retrieve List of {cls.OBJECT_NAME}",
        )

        return billable_metrics


class UpdateableAPIResource:
    @classmethod
    def Update(cls, id: str, version: int = 1, **params):
        object_name = cls.OBJECT_NAME
        path = f"/api/v{version}/{object_name}/{id}"
        api_requestor = APIRequestor()
        api_resource = api_requestor.create_request(
            request_method=Method.PUT,
            path=path,
            object=object_name,
            data=params,
            intent=f"Update {object_name}: {id}",
        )
        return api_resource


class DeletableAPIResource:
    @classmethod
    def Delete(cls, id: str, version: int = 1, **params):
        object_name = cls.OBJECT_NAME
        path = f"/api/v{version}/{object_name}/{id}"
        api_requestor = APIRequestor()
        api_resource = api_requestor.create_request(
            request_method=Method.DELETE,
            path=path,
            object=object_name,
            data=params,
            intent=f"Delete {object_name} {id}",
        )
        return api_resource



from archetypesdk.enums import Method


class APIRequestError(Exception):
    """Base class for other exceptions"""

    def __init__(self, method: Method, status_code: int, json: dict, intent: str):

        error_reason = json
        if "detail" in json:
            error_reason = json["detail"]
            if "message" in error_reason:
                error_reason = error_reason["message"]
        elif "message" in json:
            error_reason = json["message"]

        message = "Error in "
        if intent:
            message = f"{message}{intent} -- {error_reason}"
        else:
            message = f"Archetype Error: {json}"
        super().__init__(message)


class AuthRequestError(Exception):
    """The reasoning for blocking a request"""

    def __init__(self, status_code: int, json: dict, intent: str):

        error_reason = json
        if "detail" in json:
            error_reason = json["detail"]
            if "message" in error_reason:
                error_reason = error_reason["message"]
        elif "message" in json:
            error_reason = json["message"]

        message = "Error in "
        if intent:
            message = f"{message}{intent} -- {error_reason}"
        else:
            message = f"Archetype Error: {json}"
        super().__init__(message)



from __future__ import absolute_import, division, print_function
from flask import Flask, request
from datetime import datetime
import functools
import json

from archetypesdk.api_request_thread import requests_loop
from archetypesdk.auth_requestor import AuthRequestor
from archetypesdk.utils import Utils
from archetypesdk import record_auth_requests
from werkzeug.wrappers import Request, Response
import asyncio

auth_requestor = AuthRequestor()

def Auth(func):
    # request = kwargs.get("request", None)
    start_time = datetime.utcnow().timestamp()

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        args = Utils.get_request_args(request=request)
        headers = Utils.get_request_headers(request=request)
        body = Utils.get_request_body(request=request)

        header_apikey = request.headers.get("apikey", None)
        body_apikey = body.get("apikey", None)
        url_apikey = args.get("apikey", None)

        path = request.path
        method = request.method
        size = 0
        auth_response = asyncio.run_coroutine_threadsafe(auth_requestor.create_request(
            url_apikey=url_apikey,
            header_apikey=header_apikey,
            body_apikey=body_apikey,
            path=path,
            method=method,
        ), requests_loop).result()
        if auth_response.status < 400:
            return func(*args, **kwargs)

        error_reason = asyncio.run_coroutine_threadsafe(auth_response.json(), requests_loop).result()
        if "detail" in error_reason:
            error_reason = error_reason["detail"]
            if "message" in error_reason:
                error_reason = error_reason["message"]
        elif "message" in error_reason:
            error_reason = error_reason["message"]

        res = Response(
            json.dumps(error_reason),
            mimetype="application/json",
            status=auth_response.status,
        )
        return res

    return wrapper



from fastapi import Request, HTTPException
from dataclasses import dataclass
import asyncio
from archetypesdk.auth_requestor import AuthRequestor
from archetypesdk.api_request_thread import requests_loop
import json

auth_requestor = AuthRequestor()

async def authorized(request: Request) -> bool:
    args = request.query_params
    body = await request.body()
    print(body)

    if body is not None and len(body) > 0:
        body = json.loads(body)
    else:
        body = {}

    header_apikey = request.headers.get("apikey", None)
    body_apikey = body.get("apikey", None)
    url_apikey = args.get("apikey", None)

    path = request.url.path
    method = request.method
    size = 0
    auth_response = auth_requestor.create_request(
        url_apikey=url_apikey,
        header_apikey=header_apikey,
        body_apikey=body_apikey,
        path=path,
        method=method,
    )
    if auth_response.status_code < 400:
        return True

    error_reason = auth_response.json()
    if "detail" in error_reason:
        error_reason = error_reason["detail"]
        if "message" in error_reason:
            error_reason = error_reason["message"]
    elif "message" in error_reason:
        error_reason = error_reason["message"]

    raise HTTPException(status_code=401, detail=error_reason)


from __future__ import absolute_import, division, print_function

from archetypesdk.api_resources.api_resource import (
    CreatableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    RetrievableAPIResource,
)


class Product(
    CreatableAPIResource,
    RetrievableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "product"

    @classmethod
    def Retrieve(cls, product_id: str, version: int = 1):
        return super().Retrieve(id=product_id, version=version)

    @classmethod
    def All(cls, version: int = 1, **params):
        return super().All(version=version, **params)

    @classmethod
    def Create(cls, version: int = 1, **params):
        return super().Create(version=version, **params)

    @classmethod
    def Update(self, product_id: str, version: int = 1, **params):
        return super().Update(id=product_id, version=version, **params)



import re


class ClientIp:
    def __init__(self):
        pass

    @classmethod
    def is_ip(cls, value):
        if not value is None:
            ipv4 = r"^(?:(?:\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}(?:\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])$"
            ipv6 = r"^((?=.*::)(?!.*::.+::)(::)?([\dA-F]{1,4}:(:|\b)|){5}|([\dA-F]{1,4}:){6})((([\dA-F]{1,4}((?!\3)::|:\b|$))|(?!\2\3)){2}|(((2[0-4]|1\d|[1-9])?\d|25[0-5])\.?\b){4})$/i"
            return re.match(ipv4, value) or re.match(ipv6, value)

    def getClientIpFromXForwardedFor(self, value):
        try:

            if not value or value is None:
                return None

            if not isinstance(value, str):
                print("Expected a string, got -" + str(type(value)))
            else:
                # x-forwarded-for may return multiple IP addresses in the format:
                # "client IP, proxy 1 IP, proxy 2 IP"
                # Therefore, the right-most IP address is the IP address of the most recent proxy
                # and the left-most IP address is the IP address of the originating client.
                # source: http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/x-forwarded-headers.html
                # Azure Web App's also adds a port for some reason, so we'll only use the first part (the IP)
                forwardedIps = []

                for e in value.split(","):
                    ip = e.strip()
                    if ":" in ip:
                        splitted = ip.split(":")
                        if len(splitted) == 2:
                            forwardedIps.append(splitted[0])
                    forwardedIps.append(ip)

                # Sometimes IP addresses in this header can be 'unknown' (http://stackoverflow.com/a/11285650).
                # Therefore taking the left-most IP address that is not unknown
                # A Squid configuration directive can also set the value to "unknown" (http://www.squid-cache.org/Doc/config/forwarded_for/)
                return next(item for item in forwardedIps if self.is_ip(item))
        except StopIteration:
            return value.encode("utf-8")

    def get_client_address(self, environ):
        try:
            # Standard headers used by Amazon EC2, Heroku, and others.
            if "HTTP_X_CLIENT_IP" in environ:
                if self.is_ip(environ["HTTP_X_CLIENT_IP"]):
                    return environ["HTTP_X_CLIENT_IP"]

            # Load-balancers (AWS ELB) or proxies.
            if "HTTP_X_FORWARDED_FOR" in environ:
                xForwardedFor = self.getClientIpFromXForwardedFor(
                    environ["HTTP_X_FORWARDED_FOR"]
                )
                if self.is_ip(xForwardedFor):
                    return xForwardedFor

            # Cloudflare.
            # @see https://support.cloudflare.com/hc/en-us/articles/200170986-How-does-Cloudflare-handle-HTTP-Request-headers-
            # CF-Connecting-IP - applied to every request to the origin.
            if "HTTP_CF_CONNECTING_IP" in environ:
                if self.is_ip(environ["HTTP_CF_CONNECTING_IP"]):
                    return environ["HTTP_CF_CONNECTING_IP"]

            # Akamai and Cloudflare: True-Client-IP.
            if "HTTP_TRUE_CLIENT_IP" in environ:
                if self.is_ip(environ["HTTP_TRUE_CLIENT_IP"]):
                    return environ["HTTP_TRUE_CLIENT_IP"]

            # Default nginx proxy/fcgi; alternative to x-forwarded-for, used by some proxies.
            if "HTTP_X_REAL_IP" in environ:
                if self.is_ip(environ["HTTP_X_REAL_IP"]):
                    return environ["HTTP_X_REAL_IP"]

            # (Rackspace LB and Riverbed's Stingray)
            # http://www.rackspace.com/knowledge_center/article/controlling-access-to-linux-cloud-sites-based-on-the-client-ip-address
            # https://splash.riverbed.com/docs/DOC-1926
            if "HTTP_X_CLUSTER_CLIENT_IP" in environ:
                if self.is_ip(environ["HTTP_X_CLUSTER_CLIENT_IP"]):
                    return environ["HTTP_X_CLUSTER_CLIENT_IP"]

            if "HTTP_X_FORWARDED" in environ:
                if self.is_ip(environ["HTTP_X_FORWARDED"]):
                    return environ["HTTP_X_FORWARDED"]

            if "HTTP_FORWARDED_FOR" in environ:
                if self.is_ip(environ["HTTP_FORWARDED_FOR"]):
                    return environ["HTTP_FORWARDED_FOR"]

            if "HTTP_FORWARDED" in environ:
                if self.is_ip(environ["HTTP_FORWARDED"]):
                    return environ["HTTP_FORWARDED"]

            return environ["REMOTE_ADDR"]
        except KeyError:
            return environ["REMOTE_ADDR"]



import json


class Utils:
    def is_url_parsable(query: str) -> bool:
        pass

    def get_request_body(request):
        if request.method != "GET":
            try:
                request_data = request.get_data()
                fix_bytes_value = request_data.decode("unicode_escape")
                json_data = json.loads(fix_bytes_value)
                return json_data
            except Exception as e:
                return {}
        else:
            return {}

    def get_request_headers(request):
        headers = {k: v for k, v in request.headers.items()} if request.headers else {}
        return headers

    def get_request_args(request):
        args = request.args.to_dict() if request.args else {}
        return args



from __future__ import absolute_import, division, print_function

from archetypesdk.enums import Method
from archetypesdk.api_resources.api_resource import (
    CreatableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    RetrievableAPIResource,
)
from archetypesdk.api_requestor import APIRequestor
import asyncio
class BillableMetric(
    CreatableAPIResource,
    RetrievableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "billable-metric"

    @classmethod
    def Retrieve(cls, billable_metric_id: str, version: int = 1):
        return super().Retrieve(id=billable_metric_id, version=version)

    @classmethod
    def All(cls, version: int = 1, **params):
        return super().All(version=version, **params)

    @classmethod
    def Create(cls, version: int = 1, **params):
        return super().Create(version=version, **params)

    @classmethod
    def Update(self, billable_metric_id: str, version: int = 1, **params):
        return super().Update(id=billable_metric_id, version=version, **params)

    @classmethod
    def LogUsage(self, custom_uid: str, billable_metric_id: str, used_amount: float):
        data = {
            "custom_uid": custom_uid,
            "billable_metric_id": billable_metric_id,
            "used_amount": used_amount
        }
        api_requestor = APIRequestor()
        return api_requestor.create_request(
            Method.POST,
            '/sdk/v4/log-billable-metric-usage',
            data=data
        )



from __future__ import absolute_import, division, print_function


from archetypesdk.api_resources.product import Product
from archetypesdk.api_resources.endpoint import Endpoint
from archetypesdk.api_resources.billable_metric import BillableMetric
from archetypesdk.api_resources.customer import Customer
from archetypesdk.api_resources.error import APIRequestError
from archetypesdk.api_resources.auth import Auth
from archetypesdk.api_resources.token import Token



from __future__ import absolute_import, division, print_function

from archetypesdk.api_resources.api_resource import (
    CreatableAPIResource
)

class Token(
    CreatableAPIResource
):
    OBJECT_NAME = "TOKEN"


    @classmethod
    def Create(cls, custom_id: str, version: int = 1, **params):
        params["custom_id"] = custom_id
        return super().Create(version=version, **params)



import asyncio
from archetypesdk.api_resources.error import AuthRequestError
from archetypesdk.enums import Method
from archetypesdk import prod_api_base, auth_version
import requests
import json
import logging
from werkzeug.wrappers import Request, Response
from archetypesdk.api_request_thread import requests_loop


class AuthRequestor:
    def __init__(self):
        from archetypesdk import secret_key, app_id

        self.app_id = app_id
        self.secret_key = secret_key

    def create_request(
        self,
        method: str,
        path: str,
        url_apikey: str = "",
        body_apikey: str = "",
        header_apikey: str = "",
        headers: dict = {},
        data: dict = {},
        intent: str = None,
    ):
        headers = {
            "X-Archetype-SecretKey": self.secret_key,
            "X-Archetype-AppID": self.app_id,
        }

        data = {
            "path": path,
            "method": method,
            "url_apikey": url_apikey,
            "body_apikey": body_apikey,
            "header_apikey": header_apikey,
        }

        url = f"{prod_api_base}/sdk/v{auth_version}/authorize"
        logging.debug(f"POST {url}")
        response = requests.post(url=url, headers=headers, json=data)
        return response

        if response.status_code < 400:
            return response
        """
        res = Response(
            json.dumps(
                {
                    "error": "API Key not supplied. Add an 'apikey' field to the headers with your provided API key"
                }
            ),
            mimetype="text/plain",
            status=403,
        )

        
        else:
            raise AuthRequestError(
                response.status_code, response.json(), intent=intent
            )
        """



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="archetypesdk",
    version="1.0.8",
    description="Python bindings for Archetype. Archetype makes API monetization and usage based billing for APIs easy.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ArchetypeAPI/",
    author="Archetype",
    author_email="hello@archetype.dev",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.6",
    install_requires=["requests", "Flask"],
    zip_safe=False,
)


from threading import Thread
import asyncio

def _start_request_loop(loop: asyncio.AbstractEventLoop) -> None:
    asyncio.set_event_loop(loop)
    loop.run_forever()


def start_request_thread():
    loop = asyncio.new_event_loop()
    thread = Thread(target=_start_request_loop, args=(loop,), daemon=True)
    thread.start()
    return loop


requests_loop = start_request_thread()


import requests
from venv import create
from archetypesdk.api_resources.error import APIRequestError
from archetypesdk.enums import Method
from archetypesdk import prod_api_base

class APIRequestor:
    def __init__(self):
        from archetypesdk import secret_key
        from archetypesdk import app_id
        self.secret_key = secret_key
        self.app_id = app_id

    def create_request(
        self,
        request_method: Method,
        path: str,
        headers: dict = {},
        data: dict = {},
        object: str = None,
        intent: str = None,
    ):
        headers["Authorization"] = f"Bearer {self.secret_key}"
        headers["X-Archetype-AppID"] = self.app_id
        headers["X-Archetype-SecretKey"] = self.secret_key
    
        url = f"{prod_api_base}{path}"
        if request_method == Method.GET:
            response = requests.get(url=url, headers=headers)
        elif request_method == Method.POST:
            response = requests.post(url=url, headers=headers, json=data)
        elif request_method == Method.PUT:
            response = requests.put(url=url, headers=headers, json=data)
        elif request_method == Method.DELETE:
            response = requests.delete(url=url, headers=headers, json=data)

        if response.status_code < 400:
            return response.json()
        else:
            raise APIRequestError(
                request_method, response.status_code, response.json(), intent=intent
            )



from __future__ import absolute_import, division, print_function

from archetypesdk.api_resources.api_resource import (
    ListableAPIResource,
    UpdateableAPIResource,
    RetrievableAPIResource,
)
from archetypesdk.api_requestor import APIRequestor
from archetypesdk.enums import Method


class Customer(ListableAPIResource, RetrievableAPIResource, UpdateableAPIResource):
    OBJECT_NAME = "user"

    @classmethod
    def Retrieve(self, custom_uid: str, version: int = 1):
        return super().Retrieve(id=custom_uid, version=version)

    @classmethod
    def All(cls, version: int = 1, **params):
        
        return super().All(version=version, **params)

    @classmethod
    def Update(self, custom_uid: str, version: int = 1, **params):
        return super().Update(id=custom_uid, version=version, **params)

    @classmethod
    def CreateCheckoutSession(self, custom_uid: str, product_id: str, version: int = 1):
        path = f"/api/v{version}/{self.OBJECT_NAME}/{custom_uid}/checkout-session/{product_id}"
        api_requestor = APIRequestor()
        checkout_session = api_requestor.create_request(
            request_method=Method.POST,
            path=path,
            object=self.OBJECT_NAME,
            intent="Create Checkout Session",
        )
        return checkout_session

    @classmethod
    def ResetAPIKey(self, custom_uid: str, version: int = 1, **params):
        path = f"/api/v{version}/{self.OBJECT_NAME}/{custom_uid}/reset-api-key"
        
        customer_data = api_requestor.create_request(
            request_method=Method.POST,
            path=path,
            object=self.OBJECT_NAME,
            intent="Reset API Key",
        )
        return customer_data

    @classmethod
    def CreateSandboxSubscription(
        self, custom_uid: str, product_id: str, version: int = 1, **params
    ):
        path = (
            f"/api/v{version}/{self.OBJECT_NAME}/{custom_uid}/create-promo/{product_id}"
        )
        api_requestor = APIRequestor()
        sandbox_subscription = api_requestor.create_request(
            request_method=Method.POST,
            path=path,
            object=self.OBJECT_NAME,
            data=params,
            intent="Create Sandbox Subscription",
        )
        return sandbox_subscription

    @classmethod
    def CancelSubscription(self, custom_uid: str, version: int = 1):
        path = f"/api/v{version}/{self.OBJECT_NAME}/{custom_uid}/cancel-subscription"
        api_requestor = APIRequestor()
        customer_data = api_requestor.create_request(
            request_method=Method.POST,
            path=path,
            object=self.OBJECT_NAME,
            intent="Cancel Subscription",
        )
        return customer_data

    @classmethod
    def CreatePaymentLink(
        self, custom_uid: str, product_id: str, version: int = 1, **params
    ):
        path = (
            f"/api/v{version}/{self.OBJECT_NAME}/{custom_uid}/payment-link/{product_id}"
        )
        api_requestor = APIRequestor()
        payment_link_data = api_requestor.create_request(
            request_method=Method.POST,
            path=path,
            object=self.OBJECT_NAME,
            data=params,
            intent="Create Payment Link",
        )
        return payment_link_data

    @classmethod
    def Usage():
        pass

    @classmethod
    def Invoices():
        pass
